#coding=utf-8
import json

from django import forms
from django.contrib import admin
from django.core.exceptions import FieldError, ObjectDoesNotExist
from django.http import (HttpResponse, HttpResponseRedirect,
    HttpResponseBadRequest, HttpResponseForbidden)
from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.utils.html import escapejs
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _, ungettext
from django.views.decorators.csrf import csrf_exempt

from feincms.module.medialibrary.models import Category, MediaFile
from feincms.templatetags import feincms_thumbnail

from models import Gallery, GalleryMediaFile


class MediaFileWidget(forms.TextInput):
    """
    TextInput widget, shows a link to the current value if there is one.
    """

    def render(self, name, value, attrs=None):
        inputfield = super(MediaFileWidget, self).render(name, value, attrs)
        if value:
            try:
                mf = MediaFile.objects.get(pk=value)
            except MediaFile.DoesNotExist:
                return inputfield

            try:
                caption = mf.translation.caption
            except (ObjectDoesNotExist, AttributeError):
                caption = _('(no caption)')

            if mf.type == 'image':
                image = feincms_thumbnail.thumbnail(mf.file.name, '188x142')
                image = u'background: url(%(url)s) center center no-repeat;' % {'url': image}
            else:
                image = u''

            return mark_safe(u"""
                <div style="%(image)s" class="admin-gallery-image-bg absolute">
                <p class="admin-gallery-image-caption absolute">%(caption)s</p>
                %(inputfield)s</div>""" % {
                    'image': image,
                    'caption': caption,
                    'inputfield': inputfield})

        return inputfield


class ThumbnailForm(forms.Form):
    id = forms.ModelChoiceField(
        queryset=MediaFile.objects.filter(type='image')
    )
    width = forms.IntegerField(min_value=0)
    height = forms.IntegerField(min_value=0)


@csrf_exempt
def admin_thumbnail(request):
    content = u''
    if request.method == 'POST' and request.is_ajax():
        form = ThumbnailForm(request.POST)
        if not form.is_valid():
            return HttpResponseBadRequest(form.errors)
        data = form.cleaned_data

        obj = data['id']
        dimensions = '%sx%s' % (data['width'], data['height'])

        if obj.type == 'image':
            image = None
            try:
                image = feincms_thumbnail.thumbnail(obj.file.name, dimensions)
            except:
                pass

            if image:
                try:
                    caption = obj.translation.caption
                except AttributeError:
                    caption = _(u'untitled').encode('utf-8')
                content = json.dumps({
                    'url': image.url,
                    'name': escapejs(caption)
                })
        return HttpResponse(content, mimetype='application/json')
    else:
        return HttpResponseForbidden()
admin_thumbnail.short_description = _('Image')
admin_thumbnail.allow_tags = True


class MediaFileAdminForm(forms.ModelForm):
    mediafile = forms.ModelChoiceField(queryset=MediaFile.objects.filter(type='image'),
                                widget=MediaFileWidget(attrs={'class': 'image-fk'}), label=_('media file'))
    class Meta:
        model = GalleryMediaFile


class GalleryMediaFileAdmin(admin.ModelAdmin):
    form = MediaFileAdminForm
    model = GalleryMediaFile
    list_display = ['__unicode__', admin_thumbnail]
    classes = ['sortable']


class GalleryMediaFileInline(admin.StackedInline):
    model = GalleryMediaFile
    raw_id_fields = ('mediafile',)
    extra = 0
    form = MediaFileAdminForm
    classes = ['sortable']
    ordering = ['ordering']
    template = 'admin/gallery/gallery/stacked.html'


class GalleryAdmin(admin.ModelAdmin):
    inlines = (GalleryMediaFileInline,)
    list_display = ['title', 'verbose_images']

    class AddCategoryForm(forms.Form):
        _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
        category = forms.ModelChoiceField(Category.objects)

    def assign_category(self, request, queryset):
        form = None
        if 'apply' in request.POST:
            form = self.AddCategoryForm(request.POST)
            if form.is_valid():
                category = form.cleaned_data['category']
                count = 0
                mediafiles = MediaFile.objects.filter(categories=category)
                for gallery in queryset:
                    for mediafile in mediafiles:
                        try:
                            GalleryMediaFile.objects.create(gallery = gallery, mediafile=mediafile)
                        except FieldError:
                            pass
                        count += 1
                message = ungettext('Successfully added %(count)d mediafiles in %(category)s Category.',
                                    'Successfully added %(count)d mediafiles in %(category)s Categories.', count) % {
                                    'count':count, 'category':category }
                self.message_user(request, message)
                return HttpResponseRedirect(request.get_full_path())

        if not form:
            form = self.AddCategoryForm(initial={'_selected_action': request.POST.getlist(admin.ACTION_CHECKBOX_NAME)})
        return render_to_response('admin/gallery/add_category.html', {'mediafiles': queryset,
                                                         'category_form': form,
                                                        }, context_instance=RequestContext(request))
    assign_category.short_description = _('Assign Images from a Category to this Gallery')
    actions = [assign_category]


admin.site.register(Gallery, GalleryAdmin)



#coding=utf-8
from django import forms
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.paginator import Paginator, InvalidPage, EmptyPage
from django.db import models
from django.http import HttpResponse
from django.template.context import RequestContext
from django.template.loader import render_to_string
from django.utils.translation import ungettext_lazy, ugettext_lazy as _

from feincms.module.medialibrary.models import MediaFile

from .specs.legacy import DEFAULT_SPECS

__all__ = ['Gallery', 'GalleryMediaFile', 'GalleryContent', 'DEFAULT_SPECS']

class Gallery(models.Model):
    title = models.CharField(max_length=30)
    images = models.ManyToManyField(MediaFile, through='GalleryMediaFile')
    
    def ordered_images(self):
        return self.images.select_related().all()\
                                        .order_by('gallerymediafile__ordering')
    
    def count_images(self):
        if not getattr(self, '_image_count', None):
            self._image_count = self.images.count()
        return self._image_count

    def verbose_images(self):
        count = self.count_images()
        return ungettext_lazy('%(count)d Image',
                              '%(count)d Images', count) % {'count': count }
    verbose_images.short_description = _('Image Count')
    
    class Meta:
        verbose_name = _('Gallery')
        verbose_name_plural = _('Galleries')
    
    def __unicode__(self):
        return self.title
    

class GalleryMediaFile(models.Model):
    gallery = models.ForeignKey(Gallery)
    mediafile = models.ForeignKey(MediaFile)
    ordering = models.IntegerField(default=9999)
    
    class Meta:
        verbose_name = 'Image for Gallery'
        verbose_name_plural = 'Images for Gallery'
        ordering = ['ordering']
        
    def __unicode__(self):
        return u'%s' %self.mediafile


class GalleryContent(models.Model):
    @classmethod
    def initialize_type(cls, types=DEFAULT_SPECS, **kwargs):
        if 'feincms.module.medialibrary' not in settings.INSTALLED_APPS:
            raise ImproperlyConfigured, 'You have to add \'feincms.module.'\
                'medialibrary\' to your INSTALLED_APPS before creating a %s' \
                % cls.__name__

        cls.specs = dict([ ('%s_%s' % (spec.name, types.index(spec)), spec)
                                       for spec in types ])
        cls.spec_choices = [ (spec, cls.specs[spec].verbose_name )
                                       for spec in cls.specs ]

        cls.add_to_class('type', models.CharField(max_length=20,
                                 choices=cls.spec_choices,
                                 default=cls.spec_choices[0][0]))
        
    gallery = models.ForeignKey(Gallery,
        help_text=_('Choose a gallery to render here'),
        related_name='%(app_label)s_%(class)s_gallery')

    @property
    def spec(self):
        try:
            return self.specs[self.type]
        except KeyError:
            return DEFAULT_SPECS[0]

    @property
    def media(self):
        return forms.Media(**self.spec.media)
      
    def has_pagination(self):
        return self.spec.paginated
        
    class Meta:
        abstract = True
        verbose_name = _('Image Gallery')
        verbose_name_plural = _('Image Galleries')

    def process(self, request, **kwargs):
        if int(request.GET.get('gallery', 0)) == self.id and request.is_ajax():
            return HttpResponse(self.render(request, **kwargs))

    def render(self, **kwargs):
        request = kwargs.get('request')
        objects = self.gallery.ordered_images()
        remaining = []

        # check if the type is paginated
        if self.has_pagination():
            paginator = Paginator(objects, self.spec.paginate_by,
                                  orphans=self.spec.orphans)
            try:
                page = int(request.GET.get('page', 1))
            except ValueError:
                page = 1
            try:
                current_page = paginator.page(page)
            except (EmptyPage, InvalidPage):
                current_page = paginator.page(paginator.num_pages)

            images = current_page.object_list

            for object in objects:
                if object not in images:
                    remaining.append(object)
        else:
            current_page, paginator = None, None
            images = objects

        return render_to_string(self.spec.templates,
                {'content': self, 'block':current_page,
                'images': images, 'paginator': paginator,
                'remaining': remaining, 'request': request },
            context_instance = RequestContext(request))



#coding=utf-8
from django import template
import random
register = template.Library()
from django.db.models.query import QuerySet


@register.filter
def randomize(values):
    if not isinstance(values, QuerySet):
        if isinstance(values, list):
            return random.shuffle(values)
        return values
    return values.order_by('?')



try:
    from django.conf.urls import patterns, url
except ImportError:  # Older Django versions
    from django.conf.urls.defaults import patterns, url

from gallery.admin import admin_thumbnail


urlpatterns = patterns('',
    url(r'^thumbnail/$', admin_thumbnail, name='gallery_admin_thumbnail'),
)




class BaseSpec(object):
    def __init__(self, **kwargs):
        for arg in kwargs:
            setattr(self, arg, kwargs[arg])

    template_path = 'content/gallery/'
    default_template = '%sclassiclm.html' % template_path
    paginated = False

    @property
    def templates(self):
        templates = [
            '%s%s.html' % (self.template_path, self.__class__.__name__.lower()),
            self.default_template ]

        if hasattr(self, 'template_name'):
            templates.insert(0, '%s%s' % (self.template_path, self.template_name))
        return templates

    @property
    def name(self):
        return self.__class__.__name__

class Type(BaseSpec):
    pass



#!/usr/bin/env python

import os
from setuptools import setup, find_packages

setup(
	name = 'feincms-gallery',
	packages = find_packages(),
    include_package_data=True,
    version=__import__('gallery').__version__,
    description='A gallery for FeinCMS',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    author='Simon Baechler',
    author_email='sb@feinheit.ch',
    url='https://github.com/feinheit/feincms_gallery',
    license='BSD License',
    platforms=['OS Independent'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
)



""" Dowload fancybox from:
    http://fancybox.googlecode.com/files/jquery.fancybox-1.3.4.zip
"""

from django.utils.translation import ugettext_lazy as _
from .base import BaseSpec

class ClassicLightbox(BaseSpec):
    verbose_name = _('Classic Gallery with Lightbox (paginated).')
    paginated = True
    paginate_by = 12
    orphans = 3
    columns = 3
    template_name = 'p.classiclm.html'
    name = 'p.classiclm'

    media = {
        'css': {'all': ('lib/fancybox/jquery.fancybox-1.3.4.css',
                        'content/gallery/classic.css')},
        'js': ('lib/fancybox/jquery.fancybox-1.3.4.pack.js',
               'content/gallery/gallery.js'),
    }

DEFAULT_SPECS = (ClassicLightbox(),)

#TODO: write classes for all predefined gallery types:
#
#GALLERY_TYPE_CHOICES = (
#    #('p.classic' , _('Classic Gallery with full size lightbox.')),
#    #('p.clasiccaption' , _('Classic Gallery with caption and full size lightbox.')),
#    ('p.classiclm', _('Classic Gallery with Lightbox.')),
#    ('p.classiclmcaption', _('Classic Gallery with caption and Lightbox.')),
#    ('carousel', _('Single line Strip Board')),
#    ('panel', _('Fancy Panel')),
#    ('slideshow', _('Simple Slideshow')),
#    ('product', _('Product Gallery')),
#    )
#
#standard_gallery_media = {'css':{'all':('content/gallery/classic.css',)}}
#
#DEFAULT_FORM_MEDIA_DICT = {'gallery': {'css': {'all': ('lib/fancybox/jquery.fancybox-1.3.1.css', )},
#                                       'js': ('lib/fancybox/jquery.fancybox-1.3.1.pack.js',
#                                              '/media/content/gallery/gallery.js')},
#                           'gallery_p.classic': standard_gallery_media,
#                           'gallery_p.clasiccaption': standard_gallery_media,
#                           'gallery_p.classiclm': standard_gallery_media,
#                           'gallery_p.classiclmcaption': standard_gallery_media,
#                           'gallery_slideshow': {'css': {'all': ('content/gallery/slideshow.css',)},
#                                                 'js': ('content/gallery/slideshow.js',)},
#                           'gallery_carousel': {'css': {'all': ('content/gallery/carousel.css',)},
#                                                'js': ('content/gallery/jquery.infinitecarousel2.min.js',
#                                                       'content/gallery/carousel.js',)},
#                           'gallery_panel': {'js': (
#                           'content/gallery/jquery.panelgallery-2.0.0.min.js', 'content/gallery/panelgallery.js')},
#                           'gallery_product': {'css': {'all': ('content/gallery/product.css',)},
#                                               'js': ('content/gallery/jquery.infinitecarousel2.min.js',
#                                                      'content/gallery/product.js',)},
#
#                           }
